// Code generated by mocker. DO NOT EDIT.
// github.com/travisjeffery/mocker
// Source: internal/pkg/auth/login_credentials_manager.go

package mock

import (
	sync "sync"

	github_com_confluentinc_ccloud_sdk_go_v1_public "github.com/confluentinc/ccloud-sdk-go-v1-public"
	github_com_confluentinc_cli_internal_pkg_auth "github.com/confluentinc/cli/internal/pkg/auth"
	github_com_confluentinc_cli_internal_pkg_config_v1 "github.com/confluentinc/cli/internal/pkg/config/v1"
	github_com_confluentinc_cli_internal_pkg_netrc "github.com/confluentinc/cli/internal/pkg/netrc"
	github_com_spf13_cobra "github.com/spf13/cobra"
)

// LoginCredentialsManager is a mock of LoginCredentialsManager interface
type LoginCredentialsManager struct {
	lockGetCloudCredentialsFromEnvVar sync.Mutex
	GetCloudCredentialsFromEnvVarFunc func(orgResourceId string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetOnPremCredentialsFromEnvVar sync.Mutex
	GetOnPremCredentialsFromEnvVarFunc func() func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetSsoCredentialsFromConfig sync.Mutex
	GetSsoCredentialsFromConfigFunc func(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config, url string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetCredentialsFromConfig sync.Mutex
	GetCredentialsFromConfigFunc func(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config, filterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetCredentialsFromKeychain sync.Mutex
	GetCredentialsFromKeychainFunc func(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config, isCloud bool, ctxName, url string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetCredentialsFromNetrc sync.Mutex
	GetCredentialsFromNetrcFunc func(filterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetCloudCredentialsFromPrompt sync.Mutex
	GetCloudCredentialsFromPromptFunc func(orgResourceId string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetOnPremCredentialsFromPrompt sync.Mutex
	GetOnPremCredentialsFromPromptFunc func() func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetPrerunCredentialsFromConfig sync.Mutex
	GetPrerunCredentialsFromConfigFunc func(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetOnPremPrerunCredentialsFromEnvVar sync.Mutex
	GetOnPremPrerunCredentialsFromEnvVarFunc func() func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockGetOnPremPrerunCredentialsFromNetrc sync.Mutex
	GetOnPremPrerunCredentialsFromNetrcFunc func(arg0 *github_com_spf13_cobra.Command, arg1 github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error)

	lockSetCloudClient sync.Mutex
	SetCloudClientFunc func(client *github_com_confluentinc_ccloud_sdk_go_v1_public.Client)

	calls struct {
		GetCloudCredentialsFromEnvVar []struct {
			OrgResourceId string
		}
		GetOnPremCredentialsFromEnvVar []struct {
		}
		GetSsoCredentialsFromConfig []struct {
			Cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config
			Url string
		}
		GetCredentialsFromConfig []struct {
			Cfg          *github_com_confluentinc_cli_internal_pkg_config_v1.Config
			FilterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
		}
		GetCredentialsFromKeychain []struct {
			Cfg     *github_com_confluentinc_cli_internal_pkg_config_v1.Config
			IsCloud bool
			CtxName string
			Url     string
		}
		GetCredentialsFromNetrc []struct {
			FilterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
		}
		GetCloudCredentialsFromPrompt []struct {
			OrgResourceId string
		}
		GetOnPremCredentialsFromPrompt []struct {
		}
		GetPrerunCredentialsFromConfig []struct {
			Cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config
		}
		GetOnPremPrerunCredentialsFromEnvVar []struct {
		}
		GetOnPremPrerunCredentialsFromNetrc []struct {
			Arg0 *github_com_spf13_cobra.Command
			Arg1 github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
		}
		SetCloudClient []struct {
			Client *github_com_confluentinc_ccloud_sdk_go_v1_public.Client
		}
	}
}

// GetCloudCredentialsFromEnvVar mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetCloudCredentialsFromEnvVar(orgResourceId string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetCloudCredentialsFromEnvVar.Lock()
	defer m.lockGetCloudCredentialsFromEnvVar.Unlock()

	if m.GetCloudCredentialsFromEnvVarFunc == nil {
		panic("mocker: LoginCredentialsManager.GetCloudCredentialsFromEnvVarFunc is nil but LoginCredentialsManager.GetCloudCredentialsFromEnvVar was called.")
	}

	call := struct {
		OrgResourceId string
	}{
		OrgResourceId: orgResourceId,
	}

	m.calls.GetCloudCredentialsFromEnvVar = append(m.calls.GetCloudCredentialsFromEnvVar, call)

	return m.GetCloudCredentialsFromEnvVarFunc(orgResourceId)
}

// GetCloudCredentialsFromEnvVarCalled returns true if GetCloudCredentialsFromEnvVar was called at least once.
func (m *LoginCredentialsManager) GetCloudCredentialsFromEnvVarCalled() bool {
	m.lockGetCloudCredentialsFromEnvVar.Lock()
	defer m.lockGetCloudCredentialsFromEnvVar.Unlock()

	return len(m.calls.GetCloudCredentialsFromEnvVar) > 0
}

// GetCloudCredentialsFromEnvVarCalls returns the calls made to GetCloudCredentialsFromEnvVar.
func (m *LoginCredentialsManager) GetCloudCredentialsFromEnvVarCalls() []struct {
	OrgResourceId string
} {
	m.lockGetCloudCredentialsFromEnvVar.Lock()
	defer m.lockGetCloudCredentialsFromEnvVar.Unlock()

	return m.calls.GetCloudCredentialsFromEnvVar
}

// GetOnPremCredentialsFromEnvVar mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetOnPremCredentialsFromEnvVar() func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetOnPremCredentialsFromEnvVar.Lock()
	defer m.lockGetOnPremCredentialsFromEnvVar.Unlock()

	if m.GetOnPremCredentialsFromEnvVarFunc == nil {
		panic("mocker: LoginCredentialsManager.GetOnPremCredentialsFromEnvVarFunc is nil but LoginCredentialsManager.GetOnPremCredentialsFromEnvVar was called.")
	}

	call := struct {
	}{}

	m.calls.GetOnPremCredentialsFromEnvVar = append(m.calls.GetOnPremCredentialsFromEnvVar, call)

	return m.GetOnPremCredentialsFromEnvVarFunc()
}

// GetOnPremCredentialsFromEnvVarCalled returns true if GetOnPremCredentialsFromEnvVar was called at least once.
func (m *LoginCredentialsManager) GetOnPremCredentialsFromEnvVarCalled() bool {
	m.lockGetOnPremCredentialsFromEnvVar.Lock()
	defer m.lockGetOnPremCredentialsFromEnvVar.Unlock()

	return len(m.calls.GetOnPremCredentialsFromEnvVar) > 0
}

// GetOnPremCredentialsFromEnvVarCalls returns the calls made to GetOnPremCredentialsFromEnvVar.
func (m *LoginCredentialsManager) GetOnPremCredentialsFromEnvVarCalls() []struct {
} {
	m.lockGetOnPremCredentialsFromEnvVar.Lock()
	defer m.lockGetOnPremCredentialsFromEnvVar.Unlock()

	return m.calls.GetOnPremCredentialsFromEnvVar
}

// GetSsoCredentialsFromConfig mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetSsoCredentialsFromConfig(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config, url string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetSsoCredentialsFromConfig.Lock()
	defer m.lockGetSsoCredentialsFromConfig.Unlock()

	if m.GetSsoCredentialsFromConfigFunc == nil {
		panic("mocker: LoginCredentialsManager.GetSsoCredentialsFromConfigFunc is nil but LoginCredentialsManager.GetSsoCredentialsFromConfig was called.")
	}

	call := struct {
		Cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config
		Url string
	}{
		Cfg: cfg,
		Url: url,
	}

	m.calls.GetSsoCredentialsFromConfig = append(m.calls.GetSsoCredentialsFromConfig, call)

	return m.GetSsoCredentialsFromConfigFunc(cfg, url)
}

// GetSsoCredentialsFromConfigCalled returns true if GetSsoCredentialsFromConfig was called at least once.
func (m *LoginCredentialsManager) GetSsoCredentialsFromConfigCalled() bool {
	m.lockGetSsoCredentialsFromConfig.Lock()
	defer m.lockGetSsoCredentialsFromConfig.Unlock()

	return len(m.calls.GetSsoCredentialsFromConfig) > 0
}

// GetSsoCredentialsFromConfigCalls returns the calls made to GetSsoCredentialsFromConfig.
func (m *LoginCredentialsManager) GetSsoCredentialsFromConfigCalls() []struct {
	Cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config
	Url string
} {
	m.lockGetSsoCredentialsFromConfig.Lock()
	defer m.lockGetSsoCredentialsFromConfig.Unlock()

	return m.calls.GetSsoCredentialsFromConfig
}

// GetCredentialsFromConfig mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetCredentialsFromConfig(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config, filterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetCredentialsFromConfig.Lock()
	defer m.lockGetCredentialsFromConfig.Unlock()

	if m.GetCredentialsFromConfigFunc == nil {
		panic("mocker: LoginCredentialsManager.GetCredentialsFromConfigFunc is nil but LoginCredentialsManager.GetCredentialsFromConfig was called.")
	}

	call := struct {
		Cfg          *github_com_confluentinc_cli_internal_pkg_config_v1.Config
		FilterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
	}{
		Cfg:          cfg,
		FilterParams: filterParams,
	}

	m.calls.GetCredentialsFromConfig = append(m.calls.GetCredentialsFromConfig, call)

	return m.GetCredentialsFromConfigFunc(cfg, filterParams)
}

// GetCredentialsFromConfigCalled returns true if GetCredentialsFromConfig was called at least once.
func (m *LoginCredentialsManager) GetCredentialsFromConfigCalled() bool {
	m.lockGetCredentialsFromConfig.Lock()
	defer m.lockGetCredentialsFromConfig.Unlock()

	return len(m.calls.GetCredentialsFromConfig) > 0
}

// GetCredentialsFromConfigCalls returns the calls made to GetCredentialsFromConfig.
func (m *LoginCredentialsManager) GetCredentialsFromConfigCalls() []struct {
	Cfg          *github_com_confluentinc_cli_internal_pkg_config_v1.Config
	FilterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
} {
	m.lockGetCredentialsFromConfig.Lock()
	defer m.lockGetCredentialsFromConfig.Unlock()

	return m.calls.GetCredentialsFromConfig
}

// GetCredentialsFromKeychain mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetCredentialsFromKeychain(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config, isCloud bool, ctxName, url string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetCredentialsFromKeychain.Lock()
	defer m.lockGetCredentialsFromKeychain.Unlock()

	if m.GetCredentialsFromKeychainFunc == nil {
		panic("mocker: LoginCredentialsManager.GetCredentialsFromKeychainFunc is nil but LoginCredentialsManager.GetCredentialsFromKeychain was called.")
	}

	call := struct {
		Cfg     *github_com_confluentinc_cli_internal_pkg_config_v1.Config
		IsCloud bool
		CtxName string
		Url     string
	}{
		Cfg:     cfg,
		IsCloud: isCloud,
		CtxName: ctxName,
		Url:     url,
	}

	m.calls.GetCredentialsFromKeychain = append(m.calls.GetCredentialsFromKeychain, call)

	return m.GetCredentialsFromKeychainFunc(cfg, isCloud, ctxName, url)
}

// GetCredentialsFromKeychainCalled returns true if GetCredentialsFromKeychain was called at least once.
func (m *LoginCredentialsManager) GetCredentialsFromKeychainCalled() bool {
	m.lockGetCredentialsFromKeychain.Lock()
	defer m.lockGetCredentialsFromKeychain.Unlock()

	return len(m.calls.GetCredentialsFromKeychain) > 0
}

// GetCredentialsFromKeychainCalls returns the calls made to GetCredentialsFromKeychain.
func (m *LoginCredentialsManager) GetCredentialsFromKeychainCalls() []struct {
	Cfg     *github_com_confluentinc_cli_internal_pkg_config_v1.Config
	IsCloud bool
	CtxName string
	Url     string
} {
	m.lockGetCredentialsFromKeychain.Lock()
	defer m.lockGetCredentialsFromKeychain.Unlock()

	return m.calls.GetCredentialsFromKeychain
}

// GetCredentialsFromNetrc mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetCredentialsFromNetrc(filterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetCredentialsFromNetrc.Lock()
	defer m.lockGetCredentialsFromNetrc.Unlock()

	if m.GetCredentialsFromNetrcFunc == nil {
		panic("mocker: LoginCredentialsManager.GetCredentialsFromNetrcFunc is nil but LoginCredentialsManager.GetCredentialsFromNetrc was called.")
	}

	call := struct {
		FilterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
	}{
		FilterParams: filterParams,
	}

	m.calls.GetCredentialsFromNetrc = append(m.calls.GetCredentialsFromNetrc, call)

	return m.GetCredentialsFromNetrcFunc(filterParams)
}

// GetCredentialsFromNetrcCalled returns true if GetCredentialsFromNetrc was called at least once.
func (m *LoginCredentialsManager) GetCredentialsFromNetrcCalled() bool {
	m.lockGetCredentialsFromNetrc.Lock()
	defer m.lockGetCredentialsFromNetrc.Unlock()

	return len(m.calls.GetCredentialsFromNetrc) > 0
}

// GetCredentialsFromNetrcCalls returns the calls made to GetCredentialsFromNetrc.
func (m *LoginCredentialsManager) GetCredentialsFromNetrcCalls() []struct {
	FilterParams github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
} {
	m.lockGetCredentialsFromNetrc.Lock()
	defer m.lockGetCredentialsFromNetrc.Unlock()

	return m.calls.GetCredentialsFromNetrc
}

// GetCloudCredentialsFromPrompt mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetCloudCredentialsFromPrompt(orgResourceId string) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetCloudCredentialsFromPrompt.Lock()
	defer m.lockGetCloudCredentialsFromPrompt.Unlock()

	if m.GetCloudCredentialsFromPromptFunc == nil {
		panic("mocker: LoginCredentialsManager.GetCloudCredentialsFromPromptFunc is nil but LoginCredentialsManager.GetCloudCredentialsFromPrompt was called.")
	}

	call := struct {
		OrgResourceId string
	}{
		OrgResourceId: orgResourceId,
	}

	m.calls.GetCloudCredentialsFromPrompt = append(m.calls.GetCloudCredentialsFromPrompt, call)

	return m.GetCloudCredentialsFromPromptFunc(orgResourceId)
}

// GetCloudCredentialsFromPromptCalled returns true if GetCloudCredentialsFromPrompt was called at least once.
func (m *LoginCredentialsManager) GetCloudCredentialsFromPromptCalled() bool {
	m.lockGetCloudCredentialsFromPrompt.Lock()
	defer m.lockGetCloudCredentialsFromPrompt.Unlock()

	return len(m.calls.GetCloudCredentialsFromPrompt) > 0
}

// GetCloudCredentialsFromPromptCalls returns the calls made to GetCloudCredentialsFromPrompt.
func (m *LoginCredentialsManager) GetCloudCredentialsFromPromptCalls() []struct {
	OrgResourceId string
} {
	m.lockGetCloudCredentialsFromPrompt.Lock()
	defer m.lockGetCloudCredentialsFromPrompt.Unlock()

	return m.calls.GetCloudCredentialsFromPrompt
}

// GetOnPremCredentialsFromPrompt mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetOnPremCredentialsFromPrompt() func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetOnPremCredentialsFromPrompt.Lock()
	defer m.lockGetOnPremCredentialsFromPrompt.Unlock()

	if m.GetOnPremCredentialsFromPromptFunc == nil {
		panic("mocker: LoginCredentialsManager.GetOnPremCredentialsFromPromptFunc is nil but LoginCredentialsManager.GetOnPremCredentialsFromPrompt was called.")
	}

	call := struct {
	}{}

	m.calls.GetOnPremCredentialsFromPrompt = append(m.calls.GetOnPremCredentialsFromPrompt, call)

	return m.GetOnPremCredentialsFromPromptFunc()
}

// GetOnPremCredentialsFromPromptCalled returns true if GetOnPremCredentialsFromPrompt was called at least once.
func (m *LoginCredentialsManager) GetOnPremCredentialsFromPromptCalled() bool {
	m.lockGetOnPremCredentialsFromPrompt.Lock()
	defer m.lockGetOnPremCredentialsFromPrompt.Unlock()

	return len(m.calls.GetOnPremCredentialsFromPrompt) > 0
}

// GetOnPremCredentialsFromPromptCalls returns the calls made to GetOnPremCredentialsFromPrompt.
func (m *LoginCredentialsManager) GetOnPremCredentialsFromPromptCalls() []struct {
} {
	m.lockGetOnPremCredentialsFromPrompt.Lock()
	defer m.lockGetOnPremCredentialsFromPrompt.Unlock()

	return m.calls.GetOnPremCredentialsFromPrompt
}

// GetPrerunCredentialsFromConfig mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetPrerunCredentialsFromConfig(cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetPrerunCredentialsFromConfig.Lock()
	defer m.lockGetPrerunCredentialsFromConfig.Unlock()

	if m.GetPrerunCredentialsFromConfigFunc == nil {
		panic("mocker: LoginCredentialsManager.GetPrerunCredentialsFromConfigFunc is nil but LoginCredentialsManager.GetPrerunCredentialsFromConfig was called.")
	}

	call := struct {
		Cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config
	}{
		Cfg: cfg,
	}

	m.calls.GetPrerunCredentialsFromConfig = append(m.calls.GetPrerunCredentialsFromConfig, call)

	return m.GetPrerunCredentialsFromConfigFunc(cfg)
}

// GetPrerunCredentialsFromConfigCalled returns true if GetPrerunCredentialsFromConfig was called at least once.
func (m *LoginCredentialsManager) GetPrerunCredentialsFromConfigCalled() bool {
	m.lockGetPrerunCredentialsFromConfig.Lock()
	defer m.lockGetPrerunCredentialsFromConfig.Unlock()

	return len(m.calls.GetPrerunCredentialsFromConfig) > 0
}

// GetPrerunCredentialsFromConfigCalls returns the calls made to GetPrerunCredentialsFromConfig.
func (m *LoginCredentialsManager) GetPrerunCredentialsFromConfigCalls() []struct {
	Cfg *github_com_confluentinc_cli_internal_pkg_config_v1.Config
} {
	m.lockGetPrerunCredentialsFromConfig.Lock()
	defer m.lockGetPrerunCredentialsFromConfig.Unlock()

	return m.calls.GetPrerunCredentialsFromConfig
}

// GetOnPremPrerunCredentialsFromEnvVar mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetOnPremPrerunCredentialsFromEnvVar() func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetOnPremPrerunCredentialsFromEnvVar.Lock()
	defer m.lockGetOnPremPrerunCredentialsFromEnvVar.Unlock()

	if m.GetOnPremPrerunCredentialsFromEnvVarFunc == nil {
		panic("mocker: LoginCredentialsManager.GetOnPremPrerunCredentialsFromEnvVarFunc is nil but LoginCredentialsManager.GetOnPremPrerunCredentialsFromEnvVar was called.")
	}

	call := struct {
	}{}

	m.calls.GetOnPremPrerunCredentialsFromEnvVar = append(m.calls.GetOnPremPrerunCredentialsFromEnvVar, call)

	return m.GetOnPremPrerunCredentialsFromEnvVarFunc()
}

// GetOnPremPrerunCredentialsFromEnvVarCalled returns true if GetOnPremPrerunCredentialsFromEnvVar was called at least once.
func (m *LoginCredentialsManager) GetOnPremPrerunCredentialsFromEnvVarCalled() bool {
	m.lockGetOnPremPrerunCredentialsFromEnvVar.Lock()
	defer m.lockGetOnPremPrerunCredentialsFromEnvVar.Unlock()

	return len(m.calls.GetOnPremPrerunCredentialsFromEnvVar) > 0
}

// GetOnPremPrerunCredentialsFromEnvVarCalls returns the calls made to GetOnPremPrerunCredentialsFromEnvVar.
func (m *LoginCredentialsManager) GetOnPremPrerunCredentialsFromEnvVarCalls() []struct {
} {
	m.lockGetOnPremPrerunCredentialsFromEnvVar.Lock()
	defer m.lockGetOnPremPrerunCredentialsFromEnvVar.Unlock()

	return m.calls.GetOnPremPrerunCredentialsFromEnvVar
}

// GetOnPremPrerunCredentialsFromNetrc mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) GetOnPremPrerunCredentialsFromNetrc(arg0 *github_com_spf13_cobra.Command, arg1 github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams) func() (*github_com_confluentinc_cli_internal_pkg_auth.Credentials, error) {
	m.lockGetOnPremPrerunCredentialsFromNetrc.Lock()
	defer m.lockGetOnPremPrerunCredentialsFromNetrc.Unlock()

	if m.GetOnPremPrerunCredentialsFromNetrcFunc == nil {
		panic("mocker: LoginCredentialsManager.GetOnPremPrerunCredentialsFromNetrcFunc is nil but LoginCredentialsManager.GetOnPremPrerunCredentialsFromNetrc was called.")
	}

	call := struct {
		Arg0 *github_com_spf13_cobra.Command
		Arg1 github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
	}{
		Arg0: arg0,
		Arg1: arg1,
	}

	m.calls.GetOnPremPrerunCredentialsFromNetrc = append(m.calls.GetOnPremPrerunCredentialsFromNetrc, call)

	return m.GetOnPremPrerunCredentialsFromNetrcFunc(arg0, arg1)
}

// GetOnPremPrerunCredentialsFromNetrcCalled returns true if GetOnPremPrerunCredentialsFromNetrc was called at least once.
func (m *LoginCredentialsManager) GetOnPremPrerunCredentialsFromNetrcCalled() bool {
	m.lockGetOnPremPrerunCredentialsFromNetrc.Lock()
	defer m.lockGetOnPremPrerunCredentialsFromNetrc.Unlock()

	return len(m.calls.GetOnPremPrerunCredentialsFromNetrc) > 0
}

// GetOnPremPrerunCredentialsFromNetrcCalls returns the calls made to GetOnPremPrerunCredentialsFromNetrc.
func (m *LoginCredentialsManager) GetOnPremPrerunCredentialsFromNetrcCalls() []struct {
	Arg0 *github_com_spf13_cobra.Command
	Arg1 github_com_confluentinc_cli_internal_pkg_netrc.NetrcMachineParams
} {
	m.lockGetOnPremPrerunCredentialsFromNetrc.Lock()
	defer m.lockGetOnPremPrerunCredentialsFromNetrc.Unlock()

	return m.calls.GetOnPremPrerunCredentialsFromNetrc
}

// SetCloudClient mocks base method by wrapping the associated func.
func (m *LoginCredentialsManager) SetCloudClient(client *github_com_confluentinc_ccloud_sdk_go_v1_public.Client) {
	m.lockSetCloudClient.Lock()
	defer m.lockSetCloudClient.Unlock()

	if m.SetCloudClientFunc == nil {
		panic("mocker: LoginCredentialsManager.SetCloudClientFunc is nil but LoginCredentialsManager.SetCloudClient was called.")
	}

	call := struct {
		Client *github_com_confluentinc_ccloud_sdk_go_v1_public.Client
	}{
		Client: client,
	}

	m.calls.SetCloudClient = append(m.calls.SetCloudClient, call)

	m.SetCloudClientFunc(client)
}

// SetCloudClientCalled returns true if SetCloudClient was called at least once.
func (m *LoginCredentialsManager) SetCloudClientCalled() bool {
	m.lockSetCloudClient.Lock()
	defer m.lockSetCloudClient.Unlock()

	return len(m.calls.SetCloudClient) > 0
}

// SetCloudClientCalls returns the calls made to SetCloudClient.
func (m *LoginCredentialsManager) SetCloudClientCalls() []struct {
	Client *github_com_confluentinc_ccloud_sdk_go_v1_public.Client
} {
	m.lockSetCloudClient.Lock()
	defer m.lockSetCloudClient.Unlock()

	return m.calls.SetCloudClient
}

// Reset resets the calls made to the mocked methods.
func (m *LoginCredentialsManager) Reset() {
	m.lockGetCloudCredentialsFromEnvVar.Lock()
	m.calls.GetCloudCredentialsFromEnvVar = nil
	m.lockGetCloudCredentialsFromEnvVar.Unlock()
	m.lockGetOnPremCredentialsFromEnvVar.Lock()
	m.calls.GetOnPremCredentialsFromEnvVar = nil
	m.lockGetOnPremCredentialsFromEnvVar.Unlock()
	m.lockGetSsoCredentialsFromConfig.Lock()
	m.calls.GetSsoCredentialsFromConfig = nil
	m.lockGetSsoCredentialsFromConfig.Unlock()
	m.lockGetCredentialsFromConfig.Lock()
	m.calls.GetCredentialsFromConfig = nil
	m.lockGetCredentialsFromConfig.Unlock()
	m.lockGetCredentialsFromKeychain.Lock()
	m.calls.GetCredentialsFromKeychain = nil
	m.lockGetCredentialsFromKeychain.Unlock()
	m.lockGetCredentialsFromNetrc.Lock()
	m.calls.GetCredentialsFromNetrc = nil
	m.lockGetCredentialsFromNetrc.Unlock()
	m.lockGetCloudCredentialsFromPrompt.Lock()
	m.calls.GetCloudCredentialsFromPrompt = nil
	m.lockGetCloudCredentialsFromPrompt.Unlock()
	m.lockGetOnPremCredentialsFromPrompt.Lock()
	m.calls.GetOnPremCredentialsFromPrompt = nil
	m.lockGetOnPremCredentialsFromPrompt.Unlock()
	m.lockGetPrerunCredentialsFromConfig.Lock()
	m.calls.GetPrerunCredentialsFromConfig = nil
	m.lockGetPrerunCredentialsFromConfig.Unlock()
	m.lockGetOnPremPrerunCredentialsFromEnvVar.Lock()
	m.calls.GetOnPremPrerunCredentialsFromEnvVar = nil
	m.lockGetOnPremPrerunCredentialsFromEnvVar.Unlock()
	m.lockGetOnPremPrerunCredentialsFromNetrc.Lock()
	m.calls.GetOnPremPrerunCredentialsFromNetrc = nil
	m.lockGetOnPremPrerunCredentialsFromNetrc.Unlock()
	m.lockSetCloudClient.Lock()
	m.calls.SetCloudClient = nil
	m.lockSetCloudClient.Unlock()
}
